import React, { forwardRef, Ref, useEffect, useImperativeHandle } from 'react'
import { useTranslation } from 'react-i18next'
import { Controller, SubmitHandler, useForm } from 'react-hook-form'

import { Box, Stack, TextField } from '@mui/material'

import format from 'date-fns/format'
import { DatePicker } from '@mui/lab'
import { RegisterFormData } from './formData'
import { postJsonOpts } from '../../utils'
import { PersonFormMethods } from './personFormMethods'
import useOus from '../../hooks/useOus'
import useRoleTypes from '../../hooks/useRoleTypes'


interface StepSummaryProperties {
  formData: RegisterFormData
}

const StepSummary = forwardRef((props: StepSummaryProperties, ref: Ref<PersonFormMethods>) => {
    const { i18n, t } = useTranslation(['common'])
    const { formData } = props
    const ous = useOus()
    const roleTypes = useRoleTypes()

    const submit: SubmitHandler<RegisterFormData> = (data) => {
      const payload = {
        first_name: data.first_name,
        last_name: data.last_name,
        role_type: data.role_type,
        role_start: data.role_start === null ? null : format(data.role_start as Date, 'yyyy-MM-dd'),
        role_end: data.role_end === null ? null : format(data.role_end as Date, 'yyyy-MM-dd'),
        comment: data.comment,
        ou: data.ou_id,
        email: data.email,
      }
      console.log('submitting', JSON.stringify(payload))
      fetch('/api/ui/v1/register/', postJsonOpts(payload))
        .then((res) => res.text())
        .then((result) => {
          console.log('result', result)
        })
        .catch((error) => {
          console.log('error', error)
        })
    }

    const getSelectedOrganisationalUnit = () => {
      if (formData?.ou_id) {
        const ousArray = ous.filter(((value) => value.id === formData.ou_id))

        if (ousArray.length === 0) {
          return ''
        }

        if (i18n.language === 'en') {
          return ousArray[0].en
        }
        return ousArray[0].nb
      }
      return ''
    }

    const getSelectedRoleType = () => {
      if (formData?.role_type) {
        const roleTypeArray = roleTypes.filter(((value) => value.identifier === formData.role_type))

        if (roleTypeArray.length === 0) {
          return ''
        }

        if (i18n.language === 'en') {
          return roleTypeArray[0].name_en
        }
        return roleTypeArray[0].name_nb
      }
      return ''
    }

    const {
      reset,
      register,
      handleSubmit,
      control,
    } = useForm<RegisterFormData>()
    const onSubmit = handleSubmit(submit)

    useEffect(() => {
      reset(formData)
    }, [])

    function doSubmit() {
      return onSubmit()
    }

    useImperativeHandle(ref, () => ({ doSubmit }))

    return (
      <>
        <Box sx={{ typography: 'subtitle2' }}>
          {t('registerWizardText.summaryPage')}
        </Box>
        <Box sx={{ maxWidth: '30rem' }}>
          <form onSubmit={onSubmit}>
            <Stack spacing={2}>
              <TextField
                id='firstName'
                label={t('input.firstName')}
                disabled
                {...register('first_name')}
              />
              <TextField
                id='lastName'
                label={t('input.lastName')}
                disabled
                {...register('last_name')}
              />

              <TextField
                id='email'
                label={t('input.email')}
                disabled
                {...register('email')}
              />

              <TextField
                id='ou'
                value={getSelectedOrganisationalUnit()}
                label={t('ou')}
                disabled
              />

              <TextField
                id='roletype-select'
                value={getSelectedRoleType()}
                label={t('input.roleType')}
                disabled
              />

              <Controller name='role_start'
                          control={control}
                          render={({ field }) => (
                            <DatePicker
                              mask='____-__-__'
                              label={t('input.roleStartDate')}
                              disabled
                              value={field.value}
                              inputFormat='yyyy-MM-dd'
                              onChange={(value) => {
                                field.onChange(value)
                              }}
                              renderInput={(params) => <TextField {...params} />}
                            />)}
              />

              <Controller name='role_end'
                          control={control}
                          render={({ field }) => (
                            <DatePicker
                              mask='____-__-__'
                              label={t('input.roleEndDate')}
                              disabled
                              value={field.value}
                              inputFormat='yyyy-MM-dd'
                              onChange={(value) => {
                                field.onChange(value)
                              }}
                              renderInput={(params) => <TextField {...params} />}
                            />)}
              />

              <TextField
                id='comment'
                label={t('input.comment')}
                disabled
                multiline
                rows={5}
                {...register(`comment`)}
              />

            </Stack>
          </form>
        </Box>
      </>
    )
  },
)

export default StepSummary